# -*- coding: utf-8 -*-

"""
Yet another Python API to DuckDuckGo Instant Answer API https://api.duckduckgo.com/api.

Usage:
    rbd.py --help
    rbd.py QUERY...
    rbd.py [--disambiguate|--related|--json] [--url|--launch] [--save <outfile>] QUERY...
    rbd.py [--url|--launch] [--save <outfile>] [--bang <bang>] QUERY...

Option:
    -h --help               Show this screen.
    -m --disambiguate       Returns the disambiguated results.
    -r --related            Returns the related results.
    -u --url                Return the result URL.
    -l --launch             Open the result URL in a new browswer tab.
    -s --save <outfile>     Saves the JSON/HTML result to file.
    -b --bang <bang>        Returns the redirected page from DuckDuckGo !bang, see https://duckduckgo.com/bang.
    -j --json               Return the JSON responses from DuckDuckGo Instant Answer API.

Try:
    rbd.py rubberduck           # Prints: Rubberduck is a fictional character in the DC Comics universe, an anthropomorphic duck. ...
    rbd.py rubber duck          # Prints: Your query was ambiguous, try the -m option or please be more specific ...
    rbd.py -m rubber duck       # Prints: A rubber duck is a toy shaped like a stylized duck, generally yellow with a flat base. ...
    rbd.py -u rubber duck       # Prints: https://en.wikipedia.org/wiki/Rubber_duck_(disambiguation)
    rbd.py -l rubber duck       # Launch a new tab in your browser to https://en.wikipedia.org/wiki/Rubber_duck_(disambiguation)
    rbd.py -l -m rubber duck    # Launch a new tabe in your browser to https://en.wikipedia.org/wiki/Rubber_duck
    rbd.py what is the day today?            # Prints today's date.
    rbd.py what is the meaning of life?      # Prints: The meaning of life ... pertains to the significance of living or existence in general. ...
    python rbd.py -u -b wtionary rubberduck  # Prints: https://en.wiktionary.org/wiki/rubber_duck
    python rbd.py -l -b wtionary rubberduck  # Launch a new tabe to https://en.wiktionary.org/wiki/rubber_duck
    rbd.py -l -b google what is the answer to life the universe and everything         # Launch a new tab to https://encrypted.google.com/search?hl=en&q=what%20is%20the%20answer%20to%20life%20the%20universe%20and%20everything
"""

from __future__ import print_function

import io
import json
import locale
import sys
import webbrowser

from six import text_type
from docopt import docopt

import rubberduck

def print_ddg_results(ddg_results, topic_name=None):
    topic_name = topic_name if topic_name else ''
    print(topic_name, end='\n')
    for ddg_result in ddg_results:
        print ('\t'.join(['', ddg_result.url, ddg_result.text]), end='\n')
        yield '\t'.join([topic_name, ddg_result.url, ddg_result.text])


def main():
    arguments = docopt(__doc__, version='Rubberduck (rbd.py) version 0.0.1')
    query = ' '.join(arguments['QUERY'])
    # By default, return the disambiguated result based on DuckDuckGo Instant API.
    skip_disambig = 1 if arguments['--disambiguate'] else 0
    # Redirect search result when !bang is used.
    bang = arguments['--bang']
    no_redirect = 0 if bang else 1  # Always redirect when DuckDuckGo bang is used.
    response = rubberduck.search(query, bang=bang, no_redirect=no_redirect,
                                 skip_disambig=skip_disambig)
    result = response_url = response.url

    # Handles !bang requests.
    if arguments['--bang']:
        if arguments['--url']:   # Prints the redirected page.
            print(response_url)
        elif arguments['--launch']:  # Launch the redirected page.
            webbrowser.open_new_tab(response_url)
            print (response_url)
        else:  # Prints the redirected page HTML.
            if locale.getpreferredencoding(False).endswith('ASCII'):
                result = response.content
            else:
                response.content.decode('utf8')
            print (result)
            print (response.url)

    # Handles define/answer/ambiguous requests.
    else:
        ddg_json = rubberduck.DuckDuckGoJSON(response)
        response_url = ddg_json.abstract_url if ddg_json.abstract_url else response_url
        if arguments['--url']:   # Prints the redirected page.
            result = response_url
            print (response_url)
        elif arguments['--launch']:  # Launch the redirected page.
            webbrowser.open_new_tab(response_url)
            print (response_url)
        # Returns the JSON from the API response.
        elif arguments['--json']:
            result = ddg_json.pprint
            print (result)
            print (ddg_json.isambiguous, ddg_json.result_type)
        # Returns related results.
        elif ddg_json.isambiguous:
            print ("Your query was ambiguous, try the -m option or please be more specific\n")
            topics = ddg_json.ambiguous_topics
            see_also = topics.pop('See also', None)
            result = []
            # Prints the ambiguous topics.
            for topic_name, ddg_results in sorted(topics.items()):
                result+=list(print_ddg_results(ddg_results, topic_name))
            # Prints the related topics.
            result+=list(print_ddg_results(ddg_json.related_topics, 'Related'))
            # Prints the "See also" topics.
            result+=list(print_ddg_results(see_also, 'See also'))
            # Cobine the results.
            result = '\n'.join(result)
        # Returns related results.
        elif arguments['--related']:
            topics = ddg_json.related_topics
            result = []
            for ddg_result in topics:
                print ('\t'.join([ddg_result.url, ddg_result.text]), end='\n')
                result.append('\t'.join([ddg_result.url, ddg_result.text]))
            result = '\n'.join(result)
        # Returns an answer, defintiion/abstract, or "no result found".
        else:
            results_priority = [ddg_json.answer, ddg_json.abstract, u'No result found']
            for result in results_priority:
                if result:
                    break
            print(result)

    if arguments['--save']:
        with io.open(arguments['--save'], 'w', encoding='utf8') as fout:
            fout.write(result)

if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-

from __future__ import print_function
import locale
from collections import namedtuple

try: # Try to use a faster json library.
    import ujson as json
except ImportError: # Otherwise, fall back on native json.
    import json

import requests
from six import text_type


def search(query, bang=None, **kwargs):
    params = { 'format': 'json', 'pretty': '1', 'no_html': '1',
               'no_redirect': '1', 'skip_disambig': '0'}
    bangcode = '!' + bang if bang else ''
    params.update(kwargs)
    if locale.getpreferredencoding(False).endswith('ASCII'):
        query = query.decode('utf8')
    query = requests.utils.quote(text_type(query).encode('utf8'))
    url = 'http://api.duckduckgo.com/?q=' + '+'.join([bangcode, query])
    response = requests.get(url, params=params)
    return response


DuckduckgoResult = namedtuple('DuckduckgoResult', 'url, text')

def parse_results(results_json):
    for result in results_json:
        yield DuckduckgoResult(url=result['FirstURL'], text=result['Text'])

def parse_topics(topics_json):
    ambiguous_topics = {}
    related_topics = []
    for topic in topics_json:
        topic_name = topic.get('Name')
        if topic_name:
            ambiguous_topics[topic_name] = list(parse_results(topic['Topics']))
        else:
            related_topics.append(DuckduckgoResult(topic['FirstURL'], topic['Text']))
    return related_topics, ambiguous_topics


class DuckDuckGoJSON(object):
    def __init__(self, response):
        self.json = json.loads(response.content.decode('utf8'))
        self.pprint = json.dumps(self.json, indent=4)
        # Possible result types from DuckDuckGo API.
        self.response_types = {'A': 'article', 'D': 'disambiguation',
                               'C': 'category', 'N': 'name',
                               'E': 'exclusive', '': 'nothing'}
        # Result type.
        self.result_type = self.response_types[self.json['Type']]

        self.isambiguous = self.result_type == 'disambiguation'
        self.related_topics, self.ambiguous_topics =  parse_topics(self.json['RelatedTopics'])
        # Definitions.
        self.definition = self.json['Definition']
        self.definition_url = self.json['DefinitionURL']
        self.definition_source = self.json['DefinitionSource']
        # Abstract.
        self.abstract = self.json['AbstractText']
        self.abstract_url = self.json['AbstractURL']
        self.abstract_source = self.json['AbstractSource']
        self.abstract_heading = self.json['Heading']
        # Answer.
        self.answer = self.json['Answer']
        self.answer_type = self.json['AnswerType']



# -*- coding: utf-8 -*-

from setuptools import setup


entry_points = {
    'console_scripts': [
        'rbd = rbd:main',
    ]
}


requires = ['docopt', 'requests', 'six']

setup(
    # General Information
    name='rubberduck',
    version='0.0.8',
    author='Liling Tan',
    author_email='',
    description='Yet another DuckDuckGo Python API',
    url='https://github.com/alvations/rubberduck',
    license='MIT',

    # Add packages, scripts, and other data
    packages=['rubberduck'],
    scripts=['rbd.py'],

    #define requirements and entry points
    install_requires=requires,
    entry_points=entry_points,

    #pypi classifiers
    classifiers=[
         'Development Status :: 4 - Beta',
         'Environment :: Console',
         'License :: OSI Approved :: MIT License',
         'Operating System :: OS Independent',
         'Programming Language :: Python :: 2.7',
         'Programming Language :: Python :: 3.5',
         'Topic :: Internet :: WWW/HTTP',
         'Topic :: Software Development :: Libraries :: Python Modules',
    ],
)
